import enum
import pathlib
from abc import ABC, abstractmethod
from typing import Set, Union

from commonroad.planning.planning_problem import PlanningProblemSet
from commonroad.scenario.scenario import Scenario, Tag, Location


class DecimalPrecision:
    decimals = 4


precision = DecimalPrecision


class OverwriteExistingFile(enum.Enum):
    """
    Specifies whether an existing file will be overwritten or skipped
    """

    ASK_USER_INPUT = 0
    ALWAYS = 1
    SKIP = 2


class FileWriter(ABC):
    """
    Interface for writing CommonRoad files in a specific format.
    """

    def __init__(self, scenario: Scenario, planning_problem_set: PlanningProblemSet, author: str = None,
                 affiliation: str = None, source: str = None, tags: Set[Tag] = None, location: Location = None,
                 decimal_precision: int = 4):
        assert not (author is None and scenario.author is None)
        assert not (affiliation is None and scenario.affiliation is None)
        assert not (source is None and scenario.source is None)
        assert not (tags is None and scenario.tags is None)

        self.scenario: Scenario = scenario
        self.planning_problem_set: PlanningProblemSet = planning_problem_set
        self.author = author if author is not None else scenario.author
        self.affiliation = affiliation if affiliation is not None else scenario.affiliation
        self.source = source if source is not None else scenario.source
        self.location = location if location is not None else scenario.location
        self.tags = tags if tags is not None else scenario.tags

        precision.decimals = decimal_precision

    @property
    def author(self):
        return self._author

    @author.setter
    def author(self, author):
        assert isinstance(author, str), '<CommonRoadFileWriter/author> author must be a string, ' \
                                        'but has type {}'.format(type(author))
        self._author = author

    @property
    def affiliation(self):
        return self._affiliation

    @affiliation.setter
    def affiliation(self, affiliation):
        assert isinstance(affiliation, str), '<CommonRoadFileWriter/affiliation> affiliation must be a string, ' \
                                             'but has type {}'.format(type(affiliation))
        self._affiliation = affiliation

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, source):
        assert isinstance(source, str), '<CommonRoadFileWriter/source> source must be a string, ' \
                                        'but has type {}'.format(type(source))
        self._source = source

    @property
    def tags(self):
        return self._tags

    @tags.setter
    def tags(self, tags):
        for tag in tags:
            assert isinstance(tag, Tag), '<CommonRoadFileWriter/tags> tag must ' \
                                         'be a enum of type Tag, but has type {}'.format(type(tag))
        self._tags = tags

    @abstractmethod
    def _write_header(self):
        pass

    @abstractmethod
    def _add_all_objects_from_scenario(self):
        pass

    @abstractmethod
    def _add_all_planning_problems_from_planning_problem_set(self):
        pass

    @abstractmethod
    def write_to_file(self, filename: Union[str, None] = None,
                      overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT,
                      check_validity: bool = False):
        pass

    @abstractmethod
    def write_scenario_to_file(self, filename: Union[str, None] = None,
                               overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT):
        pass

    @staticmethod
    def check_validity_of_commonroad_file(commonroad_str: Union[str, bytes]) -> bool:
        pass

    @abstractmethod
    def _get_suffix(self) -> str:
        pass

    def _handle_file_path(self, filename: Union[str, None], overwrite_existing_file: OverwriteExistingFile) -> str:
        if filename is None:
            filename = str(self.scenario.scenario_id) + self._get_suffix()

        if pathlib.Path(filename).is_file():
            if overwrite_existing_file is OverwriteExistingFile.ASK_USER_INPUT:
                overwrite = input('File {} already exists, replace old file (or else skip)? (y/n)'.format(filename))
            elif overwrite_existing_file is OverwriteExistingFile.SKIP:
                overwrite = 'n'
            else:
                overwrite = 'y'

            if overwrite == 'n':
                print('Writing of file {} skipped'.format(filename))
                return ""
            else:
                print('Replace file {}'.format(filename))

        return filename



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scenario_tags.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='scenario_tags.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13scenario_tags.proto\x12\ncommonroad\"\xdd\x03\n\x07TagEnum\"\xd1\x03\n\x03Tag\x12\x0e\n\nINTERSTATE\x10\x00\x12\t\n\x05URBAN\x10\x01\x12\x0b\n\x07HIGHWAY\x10\x02\x12\x0b\n\x07\x43OMFORT\x10\x03\x12\x0c\n\x08\x43RITICAL\x10\x04\x12\x0b\n\x07\x45VASIVE\x10\x05\x12\n\n\x06\x43UT_IN\x10\x06\x12\x11\n\rILLEGAL_CUTIN\x10\x07\x12\x10\n\x0cINTERSECTION\x10\x08\x12\x0f\n\x0bLANE_CHANGE\x10\t\x12\x12\n\x0eLANE_FOLLOWING\x10\n\x12\x11\n\rMERGING_LANES\x10\x0b\x12\x0e\n\nMULTI_LANE\x10\x0c\x12\x14\n\x10ONCOMING_TRAFFIC\x10\r\x12\x17\n\x13NO_ONCOMING_TRAFFIC\x10\x0e\x12\x12\n\x0ePARALLEL_LANES\x10\x0f\x12\x0e\n\nRACE_TRACK\x10\x10\x12\x0e\n\nROUNDABOUT\x10\x11\x12\t\n\x05RURAL\x10\x12\x12\r\n\tSIMULATED\x10\x13\x12\x0f\n\x0bSINGLE_LANE\x10\x14\x12\r\n\tSLIP_ROAD\x10\x15\x12\x0f\n\x0bSPEED_LIMIT\x10\x16\x12\x0f\n\x0bTRAFFIC_JAM\x10\x17\x12\r\n\tTURN_LEFT\x10\x18\x12\x0e\n\nTURN_RIGHT\x10\x19\x12\x0c\n\x08TWO_LANE\x10\x1a\x12\x15\n\x11\x45MERGENCY_BRAKING\x10\x1b\"5\n\x0cScenarioTags\x12%\n\x04tags\x18\x01 \x03(\x0e\x32\x17.commonroad.TagEnum.Tag'
)



_TAGENUM_TAG = _descriptor.EnumDescriptor(
  name='Tag',
  full_name='commonroad.TagEnum.Tag',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INTERSTATE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='URBAN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGHWAY', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COMFORT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVASIVE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUT_IN', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ILLEGAL_CUTIN', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSECTION', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANE_CHANGE', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANE_FOLLOWING', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MERGING_LANES', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MULTI_LANE', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONCOMING_TRAFFIC', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_ONCOMING_TRAFFIC', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARALLEL_LANES', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RACE_TRACK', index=16, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROUNDABOUT', index=17, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RURAL', index=18, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIMULATED', index=19, number=19,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SINGLE_LANE', index=20, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SLIP_ROAD', index=21, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SPEED_LIMIT', index=22, number=22,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAFFIC_JAM', index=23, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TURN_LEFT', index=24, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TURN_RIGHT', index=25, number=25,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TWO_LANE', index=26, number=26,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EMERGENCY_BRAKING', index=27, number=27,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=48,
  serialized_end=513,
)
_sym_db.RegisterEnumDescriptor(_TAGENUM_TAG)


_TAGENUM = _descriptor.Descriptor(
  name='TagEnum',
  full_name='commonroad.TagEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TAGENUM_TAG,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=513,
)


_SCENARIOTAGS = _descriptor.Descriptor(
  name='ScenarioTags',
  full_name='commonroad.ScenarioTags',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tags', full_name='commonroad.ScenarioTags.tags', index=0,
      number=1, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=515,
  serialized_end=568,
)

_TAGENUM_TAG.containing_type = _TAGENUM
_SCENARIOTAGS.fields_by_name['tags'].enum_type = _TAGENUM_TAG
DESCRIPTOR.message_types_by_name['TagEnum'] = _TAGENUM
DESCRIPTOR.message_types_by_name['ScenarioTags'] = _SCENARIOTAGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TagEnum = _reflection.GeneratedProtocolMessageType('TagEnum', (_message.Message,), {
  'DESCRIPTOR' : _TAGENUM,
  '__module__' : 'scenario_tags_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TagEnum)
  })
_sym_db.RegisterMessage(TagEnum)

ScenarioTags = _reflection.GeneratedProtocolMessageType('ScenarioTags', (_message.Message,), {
  'DESCRIPTOR' : _SCENARIOTAGS,
  '__module__' : 'scenario_tags_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ScenarioTags)
  })
_sym_db.RegisterMessage(ScenarioTags)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: planning_problem.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.obstacle_pb2 as obstacle__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='planning_problem.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16planning_problem.proto\x12\ncommonroad\x1a\x0eobstacle.proto\"M\n\tGoalState\x12 \n\x05state\x18\x01 \x02(\x0b\x32\x11.commonroad.State\x12\x1e\n\x16goal_position_lanelets\x18\x02 \x03(\r\"\x84\x01\n\x0fPlanningProblem\x12\x1b\n\x13planning_problem_id\x18\x01 \x02(\r\x12(\n\rinitial_state\x18\x02 \x02(\x0b\x32\x11.commonroad.State\x12*\n\x0bgoal_states\x18\x03 \x03(\x0b\x32\x15.commonroad.GoalState'
  ,
  dependencies=[obstacle__pb2.DESCRIPTOR,])




_GOALSTATE = _descriptor.Descriptor(
  name='GoalState',
  full_name='commonroad.GoalState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='commonroad.GoalState.state', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='goal_position_lanelets', full_name='commonroad.GoalState.goal_position_lanelets', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=131,
)


_PLANNINGPROBLEM = _descriptor.Descriptor(
  name='PlanningProblem',
  full_name='commonroad.PlanningProblem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='planning_problem_id', full_name='commonroad.PlanningProblem.planning_problem_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_state', full_name='commonroad.PlanningProblem.initial_state', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='goal_states', full_name='commonroad.PlanningProblem.goal_states', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=266,
)

_GOALSTATE.fields_by_name['state'].message_type = obstacle__pb2._STATE
_PLANNINGPROBLEM.fields_by_name['initial_state'].message_type = obstacle__pb2._STATE
_PLANNINGPROBLEM.fields_by_name['goal_states'].message_type = _GOALSTATE
DESCRIPTOR.message_types_by_name['GoalState'] = _GOALSTATE
DESCRIPTOR.message_types_by_name['PlanningProblem'] = _PLANNINGPROBLEM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GoalState = _reflection.GeneratedProtocolMessageType('GoalState', (_message.Message,), {
  'DESCRIPTOR' : _GOALSTATE,
  '__module__' : 'planning_problem_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.GoalState)
  })
_sym_db.RegisterMessage(GoalState)

PlanningProblem = _reflection.GeneratedProtocolMessageType('PlanningProblem', (_message.Message,), {
  'DESCRIPTOR' : _PLANNINGPROBLEM,
  '__module__' : 'planning_problem_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.PlanningProblem)
  })
_sym_db.RegisterMessage(PlanningProblem)


# @@protoc_insertion_point(module_scope)



import datetime
import logging
import re
from typing import Tuple, List, Set, Union, Dict

import numpy as np

from commonroad.common.reader.file_reader_interface import FileReader
from commonroad.common.util import Interval, AngleInterval, Path_T, Time
from commonroad.geometry.shape import Rectangle, Circle, Polygon, Shape, ShapeGroup
from commonroad.planning.goal import GoalRegion
from commonroad.planning.planning_problem import PlanningProblemSet, PlanningProblem
from commonroad.prediction.prediction import Occupancy, TrajectoryPrediction, SetBasedPrediction
from commonroad.scenario.intersection import Intersection, IntersectionIncomingElement
from commonroad.scenario.lanelet import LaneletNetwork, Lanelet
from commonroad.common.common_lanelet import RoadUser, StopLine, LineMarking, LaneletType
from commonroad.scenario.obstacle import StaticObstacle, DynamicObstacle, EnvironmentObstacle, PhantomObstacle, \
    SignalState, ObstacleType
from commonroad.scenario.scenario import Scenario, ScenarioID, Tag, GeoTransformation, Location, Environment, \
    TimeOfDay, \
    Weather, Underground
from commonroad.scenario.state import InitialState, TraceState, CustomState, SpecificStateClasses
from commonroad.scenario.traffic_sign import TrafficSignElement, TrafficSignIDGermany, TrafficSignIDZamunda, \
    TrafficSignIDUsa, TrafficSignIDChina, TrafficSignIDSpain, TrafficSignIDRussia, TrafficSignIDArgentina, \
    TrafficSignIDBelgium, TrafficSignIDFrance, TrafficSignIDGreece, TrafficSignIDCroatia, TrafficSignIDItaly, \
    TrafficSignIDPuertoRico, TrafficSign
from commonroad.scenario.traffic_light import TrafficLightState, TrafficLightDirection, \
    TrafficLightCycleElement,  TrafficLight, TrafficLightCycle
from commonroad.scenario.trajectory import Trajectory
from commonroad.scenario_definition.protobuf_format.generated_scripts import commonroad_pb2, util_pb2, \
    phantom_obstacle_pb2
from commonroad.scenario_definition.protobuf_format.generated_scripts import lanelet_pb2, planning_problem_pb2, \
    traffic_sign_pb2, scenario_tags_pb2, environment_obstacle_pb2, obstacle_pb2, intersection_pb2, \
    dynamic_obstacle_pb2, \
    static_obstacle_pb2, traffic_light_pb2, location_pb2

logger = logging.getLogger(__name__)


class ProtobufFileReader(FileReader):
    """
    Reader for CommonRoad file stored in protobuf format.
    """

    def __init__(self, filename: Path_T):
        super().__init__(filename)

    def open(self, lanelet_assignment: bool = False) -> Tuple[Scenario, PlanningProblemSet]:
        """
        Reads a CommonRoad file in protobuf format.

        :param lanelet_assignment: Activates calculation of lanelets occupied by obstacles
        :return: Scenario and planning problems
        """
        with open(self._filename, "rb") as file:
            commonroad_msg = commonroad_pb2.CommonRoad()
            commonroad_msg.ParseFromString(file.read())

        return CommonRoadFactory.create_from_message(commonroad_msg, lanelet_assignment)

    def open_lanelet_network(self) -> LaneletNetwork:
        """
        Reads a CommonRoad file in protobuf format.

        :return: Lanelet network
        """
        with open(self._filename, "rb") as file:
            commonroad_msg = commonroad_pb2.CommonRoad()
            commonroad_msg.ParseFromString(file.read())

        scenario, _ = CommonRoadFactory.create_from_message(commonroad_msg, False)

        return scenario.lanelet_network


class CommonRoadFactory:

    @classmethod
    def create_from_message(cls, commonroad_msg: commonroad_pb2.CommonRoad, lanelet_assignment: bool) \
            -> Tuple[Scenario, PlanningProblemSet]:
        scenario_information_msg = commonroad_msg.information
        common_road_version, benchmark_id, _, author, affiliation, source, time_step_size = \
            ScenarioInformationFactory.create_from_message(scenario_information_msg)

        scenario = Scenario(time_step_size)
        scenario.scenario_id = ScenarioID.from_benchmark_id(benchmark_id, common_road_version)
        scenario.author = author
        scenario.affiliation = affiliation
        scenario.source = source

        scenario_tags = ScenarioTagsFactory.create_from_message(commonroad_msg.scenario_tags)
        scenario.tags = scenario_tags

        location = LocationFactory.create_from_message(commonroad_msg.location)
        scenario.location = location

        for lanelet_msg in commonroad_msg.lanelets:
            lanelet = LaneletFactory.create_from_message(lanelet_msg)
            scenario.add_objects(lanelet)

        for traffic_sign_msg in commonroad_msg.traffic_signs:
            traffic_sign = TrafficSignFactory.create_from_message(traffic_sign_msg)
            scenario.add_objects(traffic_sign, set())

        for traffic_light_msg in commonroad_msg.traffic_lights:
            traffic_light = TrafficLightFactory.create_from_message(traffic_light_msg)
            scenario.add_objects(traffic_light, set())

        for intersection_msg in commonroad_msg.intersections:
            intersection = IntersectionFactory.create_from_message(intersection_msg)
            scenario.add_objects(intersection)

        for static_obstacle_msg in commonroad_msg.static_obstacles:
            static_obstacle = StaticObstacleFactory \
                .create_from_message(static_obstacle_msg, scenario.lanelet_network, lanelet_assignment)
            scenario.add_objects(static_obstacle)

        for dynamic_obstacle_msg in commonroad_msg.dynamic_obstacles:
            dynamic_obstacle = DynamicObstacleFactory \
                .create_from_message(dynamic_obstacle_msg, scenario.lanelet_network, lanelet_assignment)
            scenario.add_objects(dynamic_obstacle)

        for environment_obstacle_msg in commonroad_msg.environment_obstacles:
            environment_obstacle = EnvironmentObstacleFactory.create_from_message(environment_obstacle_msg)
            scenario.add_objects(environment_obstacle)

        for phantom_obstacle_msg in commonroad_msg.phantom_obstacles:
            phantom_obstacle = PhantomObstacleFactory.create_from_message(phantom_obstacle_msg)
            scenario.add_objects(phantom_obstacle)

        planning_problem_set = PlanningProblemSet()
        for planning_problem_msg in commonroad_msg.planning_problems:
            planning_problem = PlanningProblemFactory.create_from_message(planning_problem_msg)
            planning_problem_set.add_planning_problem(planning_problem)

        return scenario, planning_problem_set


class ScenarioInformationFactory:

    @classmethod
    def create_from_message(cls, scenario_information_msg: commonroad_pb2.ScenarioInformation):
        common_road_version = scenario_information_msg.common_road_version
        benchmark_id = scenario_information_msg.benchmark_id
        date = TimeStampFactory.create_from_message(scenario_information_msg.date, cr_time=False)
        author = scenario_information_msg.author
        affiliation = scenario_information_msg.affiliation
        source = scenario_information_msg.source
        time_step_size = scenario_information_msg.time_step_size

        return common_road_version, benchmark_id, date, author, affiliation, source, time_step_size


class ScenarioTagsFactory:

    @classmethod
    def create_from_message(cls, scenario_tags_msg: scenario_tags_pb2.ScenarioTags) -> Set[Tag]:
        tags = set()
        for scenario_tag in scenario_tags_msg.tags:
            tag = scenario_tags_pb2.TagEnum.Tag.Name(scenario_tag)
            tags.add(Tag[tag])

        return tags


class LocationFactory:

    @classmethod
    def create_from_message(cls, location_msg: location_pb2.Location):
        location = Location()

        location.geo_name_id = location_msg.geo_name_id
        location.gps_latitude = location_msg.gps_latitude
        location.gps_longitude = location_msg.gps_longitude

        if location_msg.HasField('geo_transformation'):
            geo_transformation_msg = location_msg.geo_transformation
            location.geo_transformation = GeoTransformationFactory.create_from_message(geo_transformation_msg)

        if location_msg.HasField('environment'):
            environment_msg = location_msg.environment
            location.environment = EnvironmentFactory.create_from_message(environment_msg)

        return location


class GeoTransformationFactory:

    @classmethod
    def create_from_message(cls, geo_transformation_msg: location_pb2.GeoTransformation) -> GeoTransformation:
        geo_transformation = GeoTransformation()

        if geo_transformation_msg.HasField('geo_reference'):
            geo_transformation.geo_reference = geo_transformation_msg.geo_reference

        if geo_transformation_msg.HasField('x_translation'):
            geo_transformation.x_translation = geo_transformation_msg.x_translation

        if geo_transformation_msg.HasField('y_translation'):
            geo_transformation.y_translation = geo_transformation_msg.y_translation

        if geo_transformation_msg.HasField('z_rotation'):
            geo_transformation.z_rotation = geo_transformation_msg.z_rotation

        if geo_transformation_msg.HasField('scaling'):
            geo_transformation.scaling = geo_transformation_msg.scaling

        return geo_transformation


class EnvironmentFactory:

    @classmethod
    def create_from_message(cls, environment_msg: location_pb2.Environment) -> Environment:
        environment = Environment()

        if environment_msg.HasField('time'):
            environment.time = TimeStampFactory.create_from_message(environment_msg.time, cr_time=True)

        if environment_msg.HasField('time_of_day'):
            time_of_day = location_pb2.TimeOfDayEnum.TimeOfDay.Name(environment_msg.time_of_day)
            environment.time_of_day = TimeOfDay[time_of_day]

        if environment_msg.HasField('weather'):
            weather = location_pb2.WeatherEnum.Weather.Name(environment_msg.weather)
            environment.weather = Weather[weather]

        if environment_msg.HasField('underground'):
            underground = location_pb2.UndergroundEnum.Underground.Name(environment_msg.underground)
            environment.underground = Underground[underground]

        return environment


class LaneletFactory:

    @classmethod
    def create_from_message(cls, lanelet_msg: lanelet_pb2.Lanelet) -> Lanelet:
        lanelet_id = lanelet_msg.lanelet_id
        left_vertices, left_line_marking = BoundFactory.create_from_message(lanelet_msg.left_bound)
        right_vertices, right_line_marking = BoundFactory.create_from_message(lanelet_msg.right_bound)
        center_vertices = 0.5 * (left_vertices + right_vertices)

        lanelet = Lanelet(left_vertices, center_vertices, right_vertices, lanelet_id)

        predecessor = list(lanelet_msg.predecessors)
        lanelet.predecessor = predecessor

        successor = list(lanelet_msg.successors)
        lanelet.successor = successor

        if lanelet_msg.HasField('adjacent_left'):
            lanelet.adj_left = lanelet_msg.adjacent_left

        if lanelet_msg.HasField('adjacent_right'):
            lanelet.adj_right = lanelet_msg.adjacent_right

        if lanelet_msg.HasField('adjacent_left_dir'):
            left_dir = lanelet_pb2.DrivingDirEnum.DrivingDir.Name(lanelet_msg.adjacent_left_dir)
            lanelet.adj_left_same_direction = left_dir == 'SAME'

        if lanelet_msg.HasField('adjacent_right_dir'):
            right_dir = lanelet_pb2.DrivingDirEnum.DrivingDir.Name(lanelet_msg.adjacent_right_dir)
            lanelet.adj_right_same_direction = right_dir == 'SAME'

        if left_line_marking is not None:
            lanelet.line_marking_left_vertices = left_line_marking

        if right_line_marking is not None:
            lanelet.line_marking_right_vertices = right_line_marking

        if lanelet_msg.HasField('stop_line'):
            stop_line = StopLineFactory.create_from_message(lanelet_msg.stop_line)
            lanelet.stop_line = stop_line

        lanelet_types = set()
        for lanelet_type in lanelet_msg.lanelet_types:
            lanelet_types.add(LaneletType[lanelet_pb2.LaneletTypeEnum.LaneletType.Name(lanelet_type)])
        lanelet.lanelet_type = lanelet_types

        user_one_ways = set()
        for user_one_way in lanelet_msg.user_one_way:
            user_one_ways.add(RoadUser[lanelet_pb2.RoadUserEnum.RoadUser.Name(user_one_way)])
        lanelet.user_one_way = user_one_ways

        user_bidirectionals = set()
        for user_bidirectional in lanelet_msg.user_bidirectional:
            user_bidirectionals.add(RoadUser[lanelet_pb2.RoadUserEnum.RoadUser.Name(user_bidirectional)])
        lanelet.user_bidirectional = user_bidirectionals

        lanelet.traffic_signs = {ts for ts in lanelet_msg.traffic_sign_refs}
        lanelet.traffic_lights = {tl for tl in lanelet_msg.traffic_light_refs}

        return lanelet


class BoundFactory:

    @classmethod
    def create_from_message(cls, bound_msg: lanelet_pb2.Bound) -> Tuple[np.ndarray, Union[LineMarking, None]]:
        points = list()
        for point_msg in bound_msg.points:
            point = PointFactory.create_from_message(point_msg)
            points.append(point)

        line_marking = None
        if bound_msg.HasField('line_marking'):
            line_marking = LineMarking[lanelet_pb2.LineMarkingEnum.LineMarking.Name(bound_msg.line_marking)]

        return np.array(points), line_marking


class StopLineFactory:

    @classmethod
    def create_from_message(cls, stop_line_msg: lanelet_pb2.StopLine) -> StopLine:
        points = list()
        for point_msg in stop_line_msg.points:
            point = PointFactory.create_from_message(point_msg)
            points.append(point)

        line_marking = LineMarking[lanelet_pb2.LineMarkingEnum.LineMarking.Name(stop_line_msg.line_marking)]

        stop_line = StopLine(points[0], points[1], line_marking)

        traffic_sign_ref = None
        if stop_line_msg.traffic_sign_refs:
            traffic_sign_ref = {ref for ref in stop_line_msg.traffic_sign_refs}
        stop_line.traffic_sign_ref = traffic_sign_ref

        traffic_light_ref = None
        if stop_line_msg.traffic_light_refs:
            traffic_light_ref = {ref for ref in stop_line_msg.traffic_light_refs}
        stop_line.traffic_light_ref = traffic_light_ref

        return stop_line


class TrafficSignFactory:

    @classmethod
    def create_from_message(cls, traffic_sign_msg: traffic_sign_pb2.TrafficSign):
        traffic_sign_id = traffic_sign_msg.traffic_sign_id

        traffic_sign_elements = list()
        for traffic_sign_element_msg in traffic_sign_msg.traffic_sign_elements:
            traffic_sign_element = TrafficSignElementFactory.create_from_message(traffic_sign_element_msg)
            traffic_sign_elements.append(traffic_sign_element)

        point = PointFactory.create_from_message(traffic_sign_msg.position)

        traffic_sign = TrafficSign(traffic_sign_id, traffic_sign_elements, set(), point)

        if traffic_sign_msg.HasField('virtual'):
            traffic_sign.virtual = traffic_sign_msg.virtual

        return traffic_sign


class TrafficSignElementFactory:

    @classmethod
    def create_from_message(cls, traffic_sign_element_msg: traffic_sign_pb2.TrafficSignElement):
        if traffic_sign_element_msg.HasField('germany_element_id'):
            element_id = TrafficSignIDGermany[traffic_sign_pb2.TrafficSignIDGermanyEnum.TrafficSignIDGermany.Name(
                    traffic_sign_element_msg.germany_element_id)]
        elif traffic_sign_element_msg.HasField('zamunda_element_id'):
            element_id = TrafficSignIDZamunda[traffic_sign_pb2.TrafficSignIDZamundaEnum.TrafficSignIDZamunda.Name(
                    traffic_sign_element_msg.zamunda_element_id)]
        elif traffic_sign_element_msg.HasField('usa_element_id'):
            element_id = TrafficSignIDUsa[traffic_sign_pb2.TrafficSignIDUsaEnum.TrafficSignIDUsa.Name(
                    traffic_sign_element_msg.usa_element_id)]
        elif traffic_sign_element_msg.HasField('china_element_id'):
            element_id = TrafficSignIDChina[traffic_sign_pb2.TrafficSignIDChinaEnum.TrafficSignIDChina.Name(
                    traffic_sign_element_msg.china_element_id)]
        elif traffic_sign_element_msg.HasField('spain_element_id'):
            element_id = TrafficSignIDSpain[traffic_sign_pb2.TrafficSignIDSpainEnum.TrafficSignIDSpain.Name(
                    traffic_sign_element_msg.spain_element_id)]
        elif traffic_sign_element_msg.HasField('russia_element_id'):
            element_id = TrafficSignIDRussia[traffic_sign_pb2.TrafficSignIDRussiaEnum.TrafficSignIDRussia.Name(
                    traffic_sign_element_msg.russia_element_id)]
        elif traffic_sign_element_msg.HasField('argentina_element_id'):
            element_id = TrafficSignIDArgentina[traffic_sign_pb2.TrafficSignIDArgentinaEnum.TrafficSignIDArgentina.Name(
                    traffic_sign_element_msg.argentina_element_id)]
        elif traffic_sign_element_msg.HasField('belgium_element_id'):
            element_id = TrafficSignIDBelgium[traffic_sign_pb2.TrafficSignIDBelgiumEnum.TrafficSignIDBelgium.Name(
                    traffic_sign_element_msg.belgium_element_id)]
        elif traffic_sign_element_msg.HasField('france_element_id'):
            element_id = TrafficSignIDFrance[traffic_sign_pb2.TrafficSignIDFranceEnum.TrafficSignIDFrance.Name(
                    traffic_sign_element_msg.france_element_id)]
        elif traffic_sign_element_msg.HasField('greece_element_id'):
            element_id = TrafficSignIDGreece[traffic_sign_pb2.TrafficSignIDGreeceEnum.TrafficSignIDGreece.Name(
                    traffic_sign_element_msg.greece_element_id)]
        elif traffic_sign_element_msg.HasField('croatia_element_id'):
            element_id = TrafficSignIDCroatia[traffic_sign_pb2.TrafficSignIDCroatiaEnum.TrafficSignIDCroatia.Name(
                    traffic_sign_element_msg.croatia_element_id)]
        elif traffic_sign_element_msg.HasField('italy_element_id'):
            element_id = TrafficSignIDItaly[traffic_sign_pb2.TrafficSignIDItalyEnum.TrafficSignIDItaly.Name(
                    traffic_sign_element_msg.italy_element_id)]
        else:
            element_id = TrafficSignIDPuertoRico[
                traffic_sign_pb2.TrafficSignIDPuertoRicoEnum.TrafficSignIDPuertoRico.Name(
                        traffic_sign_element_msg.puerto_rico_element_id)]

        traffic_sign_element = TrafficSignElement(element_id)

        traffic_sign_element.additional_values = list(traffic_sign_element_msg.additional_values)

        return traffic_sign_element


class TrafficLightFactory:

    @classmethod
    def create_from_message(cls, traffic_light_msg: traffic_light_pb2.TrafficLight) -> TrafficLight:
        traffic_light_id = traffic_light_msg.traffic_light_id

        cycle_elements = list()
        for cycle_element_msg in traffic_light_msg.cycle_elements:
            cycle_element = CycleElementFactory.create_from_message(cycle_element_msg)
            cycle_elements.append(cycle_element)

        point = PointFactory.create_from_message(traffic_light_msg.position)

        traffic_light_cycle = TrafficLightCycle(cycle_elements)
        if traffic_light_msg.HasField('time_offset'):
            traffic_light_cycle.time_offset = traffic_light_msg.time_offset

        # extracting the color list from the traffic light cycle
        color = []
        if len(cycle_elements) > 0:
            for element in cycle_elements:
                color.append(element.state)

        traffic_light = TrafficLight(traffic_light_id, point, traffic_light_cycle, color)

        if traffic_light_msg.HasField('direction'):
            traffic_light.direction = TrafficLightDirection[
                traffic_light_pb2.TrafficLightDirectionEnum.TrafficLightDirection.Name(traffic_light_msg.direction)]

        if traffic_light_msg.HasField('active'):
            traffic_light.active = traffic_light_msg.active

        return traffic_light


class CycleElementFactory:

    @classmethod
    def create_from_message(cls, cycle_element_msg: traffic_light_pb2.CycleElement) -> TrafficLightCycleElement:
        color = TrafficLightState[traffic_light_pb2.TrafficLightStateEnum.TrafficLightState.Name(
                cycle_element_msg.color)]
        duration = cycle_element_msg.duration
        return TrafficLightCycleElement(color, duration)


class IntersectionFactory:

    @classmethod
    def create_from_message(cls, intersection_msg: intersection_pb2.Intersection) -> Intersection:
        intersection_id = intersection_msg.intersection_id

        incomings = list()
        for incoming_msg in intersection_msg.incomings:
            incoming = IncomingFactory.create_from_message(incoming_msg)
            incomings.append(incoming)

        intersection = Intersection(intersection_id, incomings)

        intersection.crossings = set(intersection_msg.crossing_lanelets)

        return intersection


class IncomingFactory:

    @classmethod
    def create_from_message(cls, incoming_msg: intersection_pb2.Incoming) -> IntersectionIncomingElement:
        incoming = IntersectionIncomingElement(incoming_msg.incoming_id)

        incoming.incoming_lanelets = set(incoming_msg.incoming_lanelets)
        incoming.successors_right = set(incoming_msg.successors_right)
        incoming.successors_straight = set(incoming_msg.successors_straight)
        incoming.successors_left = set(incoming_msg.successors_left)

        if incoming_msg.HasField('is_left_of'):
            incoming.left_of = incoming_msg.is_left_of

        return incoming


class StaticObstacleFactory:

    @classmethod
    def create_from_message(cls, static_obstacle_msg: static_obstacle_pb2.StaticObstacle,
                            lanelet_network: LaneletNetwork, lanelet_assignment: bool) -> StaticObstacle:
        static_obstacle_id = static_obstacle_msg.static_obstacle_id

        obstacle_type = ObstacleType[obstacle_pb2.ObstacleTypeEnum.ObstacleType.Name(static_obstacle_msg.obstacle_type)]

        shape = ShapeFactory.create_from_message(static_obstacle_msg.shape)

        initial_state = StateFactory.create_from_message(static_obstacle_msg.initial_state, is_initial_state=True)

        static_obstacle = StaticObstacle(static_obstacle_id, obstacle_type, shape, initial_state)

        if lanelet_assignment is True:
            rotated_shape = shape.rotate_translate_local(initial_state.position, initial_state.orientation)
            initial_shape_lanelet_ids = set(lanelet_network.find_lanelet_by_shape(rotated_shape))
            initial_center_lanelet_ids = set(lanelet_network.find_lanelet_by_position([initial_state.position])[0])
            for l_id in initial_shape_lanelet_ids:
                lanelet_network.find_lanelet_by_id(l_id).add_static_obstacle_to_lanelet(obstacle_id=static_obstacle_id)
        else:
            initial_center_lanelet_ids = None
            initial_shape_lanelet_ids = None
        static_obstacle.initial_center_lanelet_ids = initial_center_lanelet_ids
        static_obstacle.initial_shape_lanelet_ids = initial_shape_lanelet_ids

        if static_obstacle_msg.HasField('initial_signal_state'):
            initial_signal_state = SignalStateFactory.create_from_message(static_obstacle_msg.initial_signal_state)
            static_obstacle.initial_signal_state = initial_signal_state

        signal_states = list()
        for signal_state_msg in static_obstacle_msg.signal_series:
            signal_state = SignalStateFactory.create_from_message(signal_state_msg)
            signal_states.append(signal_state)
        static_obstacle.signal_series = signal_states

        return static_obstacle


class DynamicObstacleFactory:

    @classmethod
    def create_from_message(cls, dynamic_obstacle_msg: dynamic_obstacle_pb2.DynamicObstacle,
                            lanelet_network: LaneletNetwork, lanelet_assignment: bool) -> DynamicObstacle:
        dynamic_obstacle_id = dynamic_obstacle_msg.dynamic_obstacle_id

        obstacle_type = ObstacleType[
            obstacle_pb2.ObstacleTypeEnum.ObstacleType.Name(dynamic_obstacle_msg.obstacle_type)]

        shape = ShapeFactory.create_from_message(dynamic_obstacle_msg.shape)

        initial_state = StateFactory.create_from_message(dynamic_obstacle_msg.initial_state, is_initial_state=True)

        initial_center_lanelet_ids = set()
        initial_shape_lanelet_ids = set()

        prediction = None
        if dynamic_obstacle_msg.HasField('trajectory_prediction'):
            if lanelet_assignment is True:
                rotated_shape = shape.rotate_translate_local(initial_state.position, initial_state.orientation)
                initial_shape_lanelet_ids = set(lanelet_network.find_lanelet_by_shape(rotated_shape))
                initial_center_lanelet_ids = set(lanelet_network.find_lanelet_by_position([initial_state.position])[0])
                for l_id in initial_shape_lanelet_ids:
                    lanelet_network.find_lanelet_by_id(l_id) \
                        .add_dynamic_obstacle_to_lanelet(obstacle_id=dynamic_obstacle_id,
                                                         time_step=initial_state.time_step)
            else:
                initial_shape_lanelet_ids = None
                initial_center_lanelet_ids = None

            prediction = TrajectoryPredictionFactory \
                .create_from_message(dynamic_obstacle_msg.trajectory_prediction, initial_state, lanelet_network,
                                     dynamic_obstacle_id, lanelet_assignment)
        elif dynamic_obstacle_msg.HasField('set_based_prediction'):
            prediction = SetBasedPredictionFactory.create_from_message(dynamic_obstacle_msg.set_based_prediction)

        dynamic_obstacle = DynamicObstacle(dynamic_obstacle_id, obstacle_type, shape, initial_state, prediction)

        dynamic_obstacle.initial_center_lanelet_ids = initial_center_lanelet_ids
        dynamic_obstacle.initial_shape_lanelet_ids = initial_shape_lanelet_ids

        if dynamic_obstacle_msg.HasField('initial_signal_state'):
            dynamic_obstacle.initial_signal_state = SignalStateFactory \
                .create_from_message(dynamic_obstacle_msg.initial_signal_state)

        signal_states = list()
        for signal_state_msg in dynamic_obstacle_msg.signal_series:
            signal_state = SignalStateFactory.create_from_message(signal_state_msg)
            signal_states.append(signal_state)
        dynamic_obstacle.signal_series = signal_states

        return dynamic_obstacle


class EnvironmentObstacleFactory:

    @classmethod
    def create_from_message(cls, environment_obstacle_msg: environment_obstacle_pb2.EnvironmentObstacle) \
            -> EnvironmentObstacle:
        environment_obstacle_id = environment_obstacle_msg.environment_obstacle_id

        obstacle_type = ObstacleType[
            obstacle_pb2.ObstacleTypeEnum.ObstacleType.Name(environment_obstacle_msg.obstacle_type)]

        shape = ShapeFactory.create_from_message(environment_obstacle_msg.obstacle_shape)

        return EnvironmentObstacle(environment_obstacle_id, obstacle_type, shape)


class PhantomObstacleFactory:

    @classmethod
    def create_from_message(cls, phantom_obstacle_msg: phantom_obstacle_pb2.PhantomObstacle) -> PhantomObstacle:
        obstacle_id = phantom_obstacle_msg.obstacle_id

        phantom_obstacle = PhantomObstacle(obstacle_id)

        if phantom_obstacle_msg.HasField('prediction'):
            set_based_prediction = SetBasedPredictionFactory.create_from_message(phantom_obstacle_msg.prediction)
            phantom_obstacle.prediction = set_based_prediction

        return phantom_obstacle


class StateFactory:

    @classmethod
    def create_from_message(cls, state_msg: obstacle_pb2.State, is_initial_state: bool = False) -> TraceState:
        states = [state_class() for state_class in SpecificStateClasses]

        used_fields = list()
        for field in [field.name for field in state_msg.DESCRIPTOR.fields]:
            if state_msg.HasField(field):
                if field == 'point' or field == 'shape':
                    used_fields.append('position')
                else:
                    used_fields.append(field)

        matched_state = None
        for state in states:

            if len(state.attributes) != len(used_fields) and not is_initial_state:
                continue

            filled = StateFactory._fill_state(state, state_msg, state.attributes)

            if isinstance(state, InitialState) and is_initial_state:
                state.fill_with_defaults()
                return state

            if filled:
                matched_state = state
                break

        if matched_state is None:
            matched_state = CustomState()
            StateFactory._fill_state(matched_state, state_msg, used_fields)
            logger.debug("State type at time step %s cannot be matched! Creating custom state!",
                         getattr(state_msg, 'time_step'))

        return matched_state

    @classmethod
    def _fill_state(cls, state: TraceState, state_msg: obstacle_pb2.State, attrs: List[str]) -> bool:
        for attr in attrs:
            if (hasattr(state_msg, attr) and state_msg.HasField(attr)) or attr == 'position':
                if attr == 'time_step':
                    setattr(state, attr, IntegerExactOrIntervalFactory.create_from_message(state_msg.time_step))
                elif attr == 'position':
                    if state_msg.HasField('point'):
                        setattr(state, attr, PointFactory.create_from_message(state_msg.point))
                    elif state_msg.HasField('shape'):
                        setattr(state, attr, ShapeFactory.create_from_message(state_msg.shape))
                elif attr == 'orientation':
                    setattr(state, attr,
                            FloatExactOrIntervalFactory.create_from_message(state_msg.orientation, is_angle=True))
                else:
                    setattr(state, attr, FloatExactOrIntervalFactory
                            .create_from_message(getattr(state_msg, StateFactory._map_to_pb_prop(attr))))
            else:
                return False

        return True

    @staticmethod
    def _map_to_pb_prop(prop: str) -> str:
        return re.sub('(?<!^)(?=[A-Z])', '_', prop).lower()


class SignalStateFactory:

    @classmethod
    def create_from_message(cls, signal_state_msg: obstacle_pb2.SignalState) -> SignalState:
        kwargs = dict()

        for attr in SignalState.__slots__:
            if signal_state_msg.HasField(attr):
                if attr == 'time_step':
                    value = IntegerExactOrIntervalFactory.create_from_message(signal_state_msg.time_step)
                else:
                    value = getattr(signal_state_msg, attr)
                kwargs.update({attr: value})

        return SignalState(**kwargs) if kwargs else None


class OccupancyFactory:

    @classmethod
    def create_from_message(cls, occupancy_msg: obstacle_pb2.Occupancy) -> Occupancy:
        time_step = IntegerExactOrIntervalFactory.create_from_message(occupancy_msg.time_step)
        shape = ShapeFactory.create_from_message(occupancy_msg.shape)

        return Occupancy(time_step, shape)


class OccupancySetFactory:

    @classmethod
    def create_from_message(cls, occupancy_set_msg: obstacle_pb2.OccupancySet) -> List[Occupancy]:
        occupancies = list()
        for occupancy_msg in occupancy_set_msg.occupancies:
            occupancy = OccupancyFactory.create_from_message(occupancy_msg)
            occupancies.append(occupancy)

        return occupancies


class TrajectoryFactory:

    @classmethod
    def create_from_message(cls, trajectory_msg: obstacle_pb2.Trajectory) -> Trajectory:
        initial_time_step = trajectory_msg.initial_time_step

        states = list()
        for state_msg in trajectory_msg.states:
            state = StateFactory.create_from_message(state_msg)
            states.append(state)

        return Trajectory(initial_time_step, states)


class TrajectoryPredictionFactory:

    @classmethod
    def create_from_message(cls, trajectory_prediction_msg: obstacle_pb2.TrajectoryPrediction,
                            initial_state: InitialState, lanelet_network: LaneletNetwork, obstacle_id: int,
                            lanelet_assignment: bool) -> TrajectoryPrediction:
        trajectory = TrajectoryFactory.create_from_message(trajectory_prediction_msg.trajectory)

        shape = ShapeFactory.create_from_message(trajectory_prediction_msg.shape)

        trajectory_prediction = TrajectoryPrediction(trajectory, shape)

        if lanelet_assignment is True:
            shape_lanelet_assignment = cls.find_obstacle_shape_lanelets(initial_state, trajectory.state_list,
                                                                        lanelet_network, obstacle_id, shape)
            center_lanelet_assignment = cls.find_obstacle_center_lanelets(initial_state, trajectory.state_list,
                                                                          lanelet_network)
        else:
            shape_lanelet_assignment = None
            center_lanelet_assignment = None

        trajectory_prediction.center_lanelet_assignment = center_lanelet_assignment
        trajectory_prediction.shape_lanelet_assignment = shape_lanelet_assignment

        return trajectory_prediction

    @staticmethod
    def find_obstacle_shape_lanelets(initial_state: InitialState, state_list: List[TraceState],
                                     lanelet_network: LaneletNetwork,
                                     obstacle_id: int, shape: Shape) -> Dict[int, Set[int]]:

        compl_state_list = [initial_state] + state_list
        lanelet_ids_per_state = {}

        for state in compl_state_list:
            rotated_shape = shape.rotate_translate_local(state.position, state.orientation)
            lanelet_ids = lanelet_network.find_lanelet_by_shape(rotated_shape)
            for l_id in lanelet_ids:
                lanelet_network.find_lanelet_by_id(l_id).add_dynamic_obstacle_to_lanelet(obstacle_id=obstacle_id,
                                                                                         time_step=state.time_step)
            lanelet_ids_per_state[state.time_step] = set(lanelet_ids)

        return lanelet_ids_per_state

    @staticmethod
    def find_obstacle_center_lanelets(initial_state: InitialState, state_list: List[TraceState],
                                      lanelet_network: LaneletNetwork) -> Dict[int, Set[int]]:
        compl_state_list = [initial_state] + state_list
        lanelet_ids_per_state = {}

        for state in compl_state_list:
            lanelet_ids = lanelet_network.find_lanelet_by_position([state.position])[0]
            lanelet_ids_per_state[state.time_step] = set(lanelet_ids)

        return lanelet_ids_per_state


class SetBasedPredictionFactory:

    @classmethod
    def create_from_message(cls, set_based_prediction_msg: obstacle_pb2.SetBasedPrediction) -> SetBasedPrediction:
        initial_time_step = set_based_prediction_msg.initial_time_step

        occupancy_set = OccupancySetFactory.create_from_message(set_based_prediction_msg.occupancy_set)

        return SetBasedPrediction(initial_time_step, occupancy_set)


class PlanningProblemFactory:

    @classmethod
    def create_from_message(cls, planning_problem_msg: planning_problem_pb2.PlanningProblem) -> PlanningProblem:
        planning_problem_id = planning_problem_msg.planning_problem_id

        initial_state = StateFactory.create_from_message(planning_problem_msg.initial_state, is_initial_state=True)

        state_list = list()
        lanelets_of_goal_position = None
        for goal_state_msg in planning_problem_msg.goal_states:
            state, lg_position = GoalStateFactory.create_from_message(goal_state_msg)
            state_list.append(state)
            if lg_position is not None:
                if lanelets_of_goal_position is None:
                    lanelets_of_goal_position = dict()
                lanelets_of_goal_position.update({len(state_list) - 1: lg_position})

        goal_region = GoalRegion(state_list, lanelets_of_goal_position)

        return PlanningProblem(planning_problem_id, initial_state, goal_region)


class GoalStateFactory:

    @classmethod
    def create_from_message(cls, goal_state_msg: planning_problem_pb2.GoalState) -> Tuple[TraceState, List[int]]:
        state = StateFactory.create_from_message(goal_state_msg.state)

        goal_position_lanelets = None
        if goal_state_msg.goal_position_lanelets:
            goal_position_lanelets = list(goal_state_msg.goal_position_lanelets)

        return state, goal_position_lanelets


class PointFactory:

    @classmethod
    def create_from_message(cls, point_msg: util_pb2.Point) -> np.ndarray:
        return np.array([point_msg.x, point_msg.y])


class RectangleFactory:

    @classmethod
    def create_from_message(cls, rectangle_msg: util_pb2.Rectangle) -> Rectangle:
        rectangle = Rectangle(rectangle_msg.length, rectangle_msg.width)

        if rectangle_msg.HasField('center'):
            point = PointFactory.create_from_message(rectangle_msg.center)
            rectangle.center = point

        if rectangle_msg.HasField('orientation'):
            rectangle.orientation = rectangle_msg.orientation

        return rectangle


class CircleFactory:

    @classmethod
    def create_from_message(cls, circle_msg: util_pb2.Circle) -> Circle:
        circle = Circle(circle_msg.radius)

        if circle_msg.HasField('center'):
            point = PointFactory.create_from_message(circle_msg.center)
            circle.center = point

        return circle


class PolygonFactory:

    @classmethod
    def create_from_message(cls, polygon_msg: util_pb2.Polygon) -> Polygon:
        vertices = list()
        for point_msg in polygon_msg.vertices:
            point = PointFactory.create_from_message(point_msg)
            vertices.append(point)
        polygon = Polygon(np.array(vertices))

        return polygon


class ShapeGroupFactory:

    @classmethod
    def create_from_message(cls, shape_group_msg: util_pb2.ShapeGroup) -> ShapeGroup:
        shapes = list()
        for shape_msg in shape_group_msg.shapes:
            shape = ShapeFactory.create_from_message(shape_msg)
            shapes.append(shape)
        shape_group = ShapeGroup(shapes)

        return shape_group


class ShapeFactory:

    @classmethod
    def create_from_message(cls, shape_msg: util_pb2.Shape) -> Shape:
        if shape_msg.HasField('rectangle'):
            shape = RectangleFactory.create_from_message(shape_msg.rectangle)
        elif shape_msg.HasField('circle'):
            shape = CircleFactory.create_from_message(shape_msg.circle)
        elif shape_msg.HasField('polygon'):
            shape = PolygonFactory.create_from_message(shape_msg.polygon)
        else:
            shape = ShapeGroupFactory.create_from_message(shape_msg.shape_group)

        return shape


class IntegerIntervalFactory:

    @classmethod
    def create_from_message(cls, integer_interval_msg: util_pb2.IntegerInterval) -> Interval:
        return Interval(integer_interval_msg.start, integer_interval_msg.end)


class FloatIntervalFactory:

    @classmethod
    def create_from_message(cls, float_interval_msg: util_pb2.FloatInterval, is_angle: bool = False) -> Interval:
        if is_angle:
            interval = AngleInterval(float_interval_msg.start, float_interval_msg.end)
        else:
            interval = Interval(float_interval_msg.start, float_interval_msg.end)
        return interval


class IntegerExactOrIntervalFactory:

    @classmethod
    def create_from_message(cls, integer_exact_or_interval_msg: util_pb2.IntegerExactOrInterval) \
            -> Union[int, Interval]:
        if integer_exact_or_interval_msg.HasField('exact'):
            return integer_exact_or_interval_msg.exact

        if integer_exact_or_interval_msg.HasField('interval'):
            return IntegerIntervalFactory.create_from_message(integer_exact_or_interval_msg.interval)


class FloatExactOrIntervalFactory:

    @classmethod
    def create_from_message(cls, float_exact_or_interval_msg: util_pb2.FloatExactOrInterval, is_angle: bool = False) \
            -> Union[float, Interval]:
        if float_exact_or_interval_msg.HasField('exact'):
            return float_exact_or_interval_msg.exact

        if float_exact_or_interval_msg.HasField('interval'):
            return FloatIntervalFactory.create_from_message(float_exact_or_interval_msg.interval, is_angle)


class IntegerListFactory:

    @classmethod
    def create_from_message(cls, integer_list_msg: util_pb2.IntegerList) -> List[int]:
        return list(integer_list_msg)


class FloatListFactory:

    @classmethod
    def create_from_message(cls, float_list_msg: util_pb2.FloatList) -> List[float]:
        return list(float_list_msg)


class TimeStampFactory:

    @classmethod
    def create_from_message(cls, time_stamp_msg: util_pb2.TimeStamp, cr_time: bool) -> Union[datetime.datetime, Time]:
        year = 2022
        month = day = 1
        minute = hour = 0

        if time_stamp_msg.HasField('year'):
            year = time_stamp_msg.year
        if time_stamp_msg.HasField('month'):
            month = time_stamp_msg.month
        if time_stamp_msg.HasField('day'):
            day = time_stamp_msg.day
        if time_stamp_msg.HasField('hour'):
            hour = time_stamp_msg.hour
        if time_stamp_msg.HasField('minute'):
            minute = time_stamp_msg.minute

        return Time(hour, minute) if cr_time else datetime.datetime(year, month, day, hour, minute)



from abc import ABC, abstractmethod
from typing import Optional

from commonroad.visualization.renderer import IRenderer
from commonroad.visualization.draw_params import BaseParam


class IDrawable(ABC):
    """
    Interface for drawable types
    """

    @abstractmethod
    def draw(self, renderer: IRenderer, draw_params: Optional[BaseParam]) -> None:
        """
        Draw the object

        :param renderer: Renderer to use for drawing
        :param draw_params: optional parameters for plotting, overriding the parameters of the renderer
        :return: None
        """
        pass



import math
from typing import List

import numpy as np
from shapely.geometry import LineString

from commonroad.common.validity import is_valid_polyline


def compute_polyline_lengths(polyline: np.ndarray) -> np.ndarray:
    """
    Computes the path lengths of a given polyline in steps travelled
    from initial to final coordinate.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Path lengths of the polyline for each coordinate in m
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)

    distance = [0]
    for i in range(1, len(polyline)):
        distance.append(distance[i - 1] + np.linalg.norm(polyline[i] - polyline[i - 1]))

    return np.array(distance)


def compute_total_polyline_length(polyline: np.ndarray) -> float:
    """
    Computes the complete path length of a given polyline.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Path length of the polyline [m]
    """
    lengths = compute_polyline_lengths(polyline)

    return float(lengths[-1])


def compute_polyline_curvatures(polyline: np.ndarray) -> np.ndarray:
    """
    Computes the curvatures along a given polyline travelled from initial
    to final coordinate.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Curvatures of the polyline for each coordinate [1/rad]
    """
    assert is_valid_polyline(polyline) and len(polyline) >= 3, "Polyline p={} is malformed!".format(polyline)

    x_d = np.gradient(polyline[:, 0])
    x_dd = np.gradient(x_d)
    y_d = np.gradient(polyline[:, 1])
    y_dd = np.gradient(y_d)

    return (x_d * y_dd - x_dd * y_d) / ((x_d ** 2 + y_d ** 2) ** (3. / 2.))


def compute_polyline_orientations(polyline: np.ndarray) -> np.ndarray:
    """
    Computes the orientation of a given polyline travelled from initial
    to final coordinate. The orientation of the last coordinate is always
    assigned with the computed orientation of the penultimate one.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Orientations of the polyline for each coordinate [rad]
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)

    orientation = []
    for i in range(0, len(polyline) - 1):
        pt_1 = polyline[i]
        pt_2 = polyline[i + 1]
        tmp = pt_2 - pt_1
        orient = np.arctan2(tmp[1], tmp[0])
        orientation.append(orient)
        if i == len(polyline) - 2:
            orientation.append(orient)

    return np.array(orientation)


def compute_polyline_initial_orientation(polyline: np.ndarray) -> float:
    """
    Computes the orientation of the initial coordinate with respect to the succeeding
    coordinate.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Orientation of the initial coordinate [rad]
    """
    orientations = compute_polyline_orientations(polyline)

    return orientations[0]


def is_point_on_polyline(polyline: np.ndarray, point: np.ndarray) -> bool:
    """
    Computes whether a given point lies on a polyline. That means, the point is between the starting
    and ending point of the polyline.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param point: 2D point [x, y]
    :return: Boolean indicating whether point lies on polyline or not
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)

    for i in range(0, len(polyline) - 1):
        l_x_1 = polyline[i][0]
        l_y_1 = polyline[i][1]
        l_x_2 = polyline[i + 1][0]
        l_y_2 = polyline[i + 1][1]

        cross_product = (point[1] - l_y_1) * (l_x_2 - l_x_1) - (point[0] - l_x_1) * (l_y_2 - l_y_1)

        epsilon = 1e-12
        if abs(cross_product) > epsilon:
            continue

        dot_product = (point[0] - l_x_1) * (l_x_2 - l_x_1) + (point[1] - l_y_1) * (l_y_2 - l_y_1)
        if dot_product < 0:
            continue

        squared_length = (l_x_2 - l_x_1) * (l_x_2 - l_x_1) + (l_y_2 - l_y_1) * (l_y_2 - l_y_1)
        if dot_product > squared_length:
            continue

        return True

    return False


def compute_polyline_intersections(polyline_1: np.ndarray, polyline_2: np.ndarray) -> np.ndarray:
    """
    Computes the intersection points of two polylines.

    :param polyline_1: First polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param polyline_2: Second polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Intersection points
    """
    assert is_valid_polyline(polyline_1), "First polyline p={} is malformed!".format(polyline_1)
    assert is_valid_polyline(polyline_2), "Second polyline p={} is malformed!".format(polyline_2)

    intersection_points = []

    for i in range(0, len(polyline_1) - 1):
        for j in range(0, len(polyline_2) - 1):

            x_diff = [polyline_1[i][0] - polyline_1[i + 1][0], polyline_2[j][0] - polyline_2[j + 1][0]]
            y_diff = [polyline_1[i][1] - polyline_1[i + 1][1], polyline_2[j][1] - polyline_2[j + 1][1]]

            div = np.linalg.det(np.array([x_diff, y_diff]))
            if math.isclose(div, 0.0):
                continue

            d_1 = np.linalg.det(
                np.array([[polyline_1[i][0], polyline_1[i][1]], [polyline_1[i + 1][0], polyline_1[i + 1][1]]]))
            d_2 = np.linalg.det(
                np.array([[polyline_2[j][0], polyline_2[j][1]], [polyline_2[j + 1][0], polyline_2[j + 1][1]]]))
            d = [d_1, d_2]
            x = np.linalg.det(np.array([d, x_diff])) / div
            y = np.linalg.det(np.array([d, y_diff])) / div

            point = np.array([x, y])
            between_polyline_1 = is_point_on_polyline(np.array([polyline_1[i], polyline_1[i + 1]]), point)
            between_polyline_2 = is_point_on_polyline(np.array([polyline_2[j], polyline_2[j + 1]]), point)
            if [x, y] not in intersection_points and between_polyline_1 and between_polyline_2:
                intersection_points.append([x, y])

    return np.array(intersection_points)


def is_polyline_self_intersection(polyline: np.ndarray) -> bool:
    """
    Computes whether the given polyline contains self-intersection. Intersection
    at boundary points are considered as self-intersection.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :return: Self-intersection or not
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)

    line = [(x, y) for x, y in polyline]
    line_string = LineString(line)

    return not line_string.is_simple


def compare_polylines_equality(polyline_1: np.ndarray, polyline_2: np.ndarray, threshold=1e-10) -> bool:
    """
    Compares two polylines for equality. For equality of the values a threshold can be given.

    :param polyline_1: First polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param polyline_2: Second polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param threshold: Threshold for equality of values
    :return: Equality of both polylines or not
    """
    assert is_valid_polyline(polyline_1), "First polyline p={} is malformed!".format(polyline_1)
    assert is_valid_polyline(polyline_2), "Second polyline p={} is malformed!".format(polyline_2)

    return np.allclose(polyline_1, polyline_2, rtol=threshold, atol=threshold)


def resample_polyline_with_number(polyline: np.ndarray, number: int) -> np.ndarray:
    """
    Resamples the given polyline with a fixed number of points. The number
    of coordinates can be resampled down or up.
    There exists also an efficient C++ implementation with Python interface in the CommonRoad Drivability Checker.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param number: Fixed number of 2D points
    :return: Resampled polyline
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)
    assert number > 1, 'Number n={} has to be at least two'.format(number)

    line = LineString(polyline)
    dists = np.linspace(0, line.length, number)
    polyline_resampled = []
    for d in dists:
        point = line.interpolate(d)
        polyline_resampled.append([point.x, point.y])

    return np.array(polyline_resampled)


def resample_polyline_with_distance(polyline: np.ndarray, distance: float) -> np.ndarray:
    """
    Resamples the given polyline with a specific distance. For a higher distance than the length
    of the given polyline the polyline is not resampled.
    There exists also an efficient C++ implementation with Python interface in the CommonRoad Drivability Checker.

    :param polyline: Polyline with 2D points [[x_0, y_0], [x_1, y_1], ...]
    :param distance: Specific distance [m]
    :return: Resampled polyline
    """
    assert is_valid_polyline(polyline), "Polyline p={} is malformed!".format(polyline)
    assert distance > 0, 'Distance d={} has to be greater than 0'.format(distance)

    line = LineString(polyline)
    dists = np.arange(0, line.length, distance)
    polyline_resampled = []
    for d in dists:
        point = line.interpolate(d)
        polyline_resampled.append([point.x, point.y])

    last_point = polyline[-1]
    polyline_resampled.append(last_point)

    return np.array(polyline_resampled)


def equalize_polyline_length(long_polyline: np.ndarray, short_polyline: np.ndarray) -> np.ndarray:
    """
    Inserts vertices into a polyline to be of the same length than other polyline.

    :param long_polyline: Polyline with higher number of vertices
    :param short_polyline: Polyline with lower number of vertices
    :return: Short polyline with equal length as long polyline
    """
    assert is_valid_polyline(long_polyline), "Long polyline p={} is malformed!".format(long_polyline)
    assert is_valid_polyline(short_polyline), "Short polyline p={} is malformed!".format(short_polyline)
    assert len(long_polyline) > len(short_polyline), "The number of vertices of long polyline p={} must be greater " \
                                                     "compared to short polyline p={}!".format(long_polyline,
                                                                                               short_polyline)

    path_length_long = compute_polyline_lengths(long_polyline)
    path_length_percentage_long = path_length_long / path_length_long[-1]
    if len(short_polyline) > 2:
        path_length_short = compute_polyline_lengths(short_polyline)
        path_length_percentage_short = path_length_short / path_length_short[-1]
    else:
        path_length_percentage_short = [0, 1]

    index_mapping = create_indices_mapping(path_length_percentage_long, path_length_percentage_short)

    org_polyline = short_polyline
    last_key = 0
    counter = 0
    for key, value in enumerate(index_mapping):
        if value == -1:
            counter += 1
        elif counter > 0 and value > -1:
            ub = org_polyline[value]
            lb = short_polyline[last_key]
            for idx in range(1, counter + 1):
                insertion_factor = \
                    (path_length_percentage_long[last_key + idx] - path_length_percentage_long[last_key]) / \
                    (path_length_percentage_long[key] - path_length_percentage_long[last_key])
                new_vertex = insertion_factor * (ub - lb) + lb
                short_polyline_updated = np.insert(short_polyline, last_key + idx, new_vertex, 0)
                short_polyline = short_polyline_updated
            last_key = key
            counter = 0
        else:
            last_key = key
    return short_polyline


def create_indices_mapping(path_length_percentage_long: np.ndarray,
                           path_length_percentage_short: np.ndarray) -> List[int]:
    """
    Extracts places (indices) where new vertices have to be added in shorter polyline.
    Helper function for insert_vertices

    :param path_length_percentage_long: Proportional path length along longer polyline
    :param path_length_percentage_short: Proportional path length along shorter polyline
    :return: Mapping of existing indices of shorter polyline to longer polyline
    """
    path_length_percentages = [path_length_percentage_long, path_length_percentage_short]
    path_length_percentage_names = ['Long path length percentage', 'Short path length percentage']
    for i in range(0, len(path_length_percentages)):
        path_length_percentage = path_length_percentages[i]
        path_length_percentage_name = path_length_percentage_names[i]
        valid_path_percentage = True
        p = 0
        for percentage in path_length_percentage:
            if p > percentage:
                valid_path_percentage = False
                break
            p = percentage
        assert valid_path_percentage and len(path_length_percentage) > 1, \
            str(path_length_percentage_name) + " p={} is malformed!".format(path_length_percentage)
    assert len(path_length_percentage_long) > len(path_length_percentage_short), \
        "The number of long path length percentage p={} must be greater compared to the number of short path length " \
        "percentage p={}!".format(path_length_percentage_long, path_length_percentage_short)

    index_mapping = [-1] * len(path_length_percentage_long)
    index_mapping[0] = 0
    index_mapping[-1] = len(path_length_percentage_short) - 1

    finished = False

    last_idx_long = 1
    for key in range(1, len(path_length_percentage_short) - 1):
        value = path_length_percentage_short[key]
        threshold = 0.01
        while key not in index_mapping and not finished:
            for idx_long in range(last_idx_long,
                                  len(path_length_percentage_long) - (len(path_length_percentage_short) - key) + 1):
                if abs(path_length_percentage_long[idx_long] - value) < threshold and index_mapping[idx_long] == -1:
                    index_mapping[idx_long] = key
                    last_idx_long = idx_long
                    if len(path_length_percentage_short) - key + 1 == len(index_mapping) - idx_long + 1:
                        for idx in range(idx_long + 1, len(index_mapping)):
                            index_mapping[idx] = key + 1
                            key += 1
                        finished = True
                    break
            threshold *= 2
        if finished:
            break

    return index_mapping


def concatenate_polylines(head: np.ndarray, tail: np.ndarray) -> np.ndarray:
    """
    Concatenates two polylines. The head represents the first part of the new polyline.

    :param head: First part of new polyline
    :param tail: Second part of new polyline
    """
    assert is_valid_polyline(head), "Left polyline p={} is malformed!".format(head)
    assert is_valid_polyline(tail), "Right polyline p={} is malformed!".format(tail)

    return np.concatenate((head, tail), axis=0)



import enum

import enum
from pathlib import Path

import numpy as np
import math
from scipy.spatial.transform.rotation import Rotation
from scipy.spatial.transform import Slerp
from typing import Tuple
from commonroad.common import validity
from commonroad.common.validity import *

Path_T = Union[str, bytes, Path]


def interpolate_angle(x: Union[float, np.array], xp: np.array, fp: np.array, degrees=False):
    """
    :param x: The x-coordinates at which to evaluate the interpolated values.
    :param xp: The x-coordinates of the data points.
    :param fp: The y-coordinates (angles) of the data points, same length as xp.
    :param degrees: True if the input and returned angles are in degrees
    :return: The interpolated angles in radian, same shape as x.
    """

    rotations = Rotation.from_euler("z", fp, degrees=degrees)
    slerp = Slerp(xp, rotations)
    return slerp(x).as_euler("zxy", degrees=degrees)[0]


def subtract_orientations(lhs, rhs):
    """Return the signed difference between angles lhs and rhs
    :param lhs: lhs of the subtraction
    :param rhs: rhs of the subtraction
    :return: ``(lhs - rhs)``, the value will be within ``[-math.pi, math.pi)``.
    Both ``lhs`` and ``rhs`` may either be zero-based (within
    ``[0, 2*math.pi]``), or ``-pi``-based (within ``[-math.pi, math.pi]``).
    """

    return math.fmod((lhs - rhs) + math.pi * 3.0, 2.0 * math.pi) - math.pi


def make_valid_orientation(angle: float) -> float:
    while angle > TWO_PI:
        angle = angle - TWO_PI
    while angle < -TWO_PI:
        angle = angle + TWO_PI
    return angle


def make_valid_orientation_interval(angle_start: float, angle_end: float) -> Tuple[float, float]:
    while angle_start > TWO_PI or angle_end > TWO_PI:
        angle_start -= TWO_PI
        angle_end -= TWO_PI
    while angle_start < -TWO_PI or angle_start < -TWO_PI:
        angle_start += TWO_PI
        angle_end += TWO_PI
    return angle_start, angle_end


def vectorized_angle_difference(lhs: float, rhs: float) -> float:
    """
    Calculates the difference between given two angles by calculating
    the arctan2 between the unit vectors with the given angles.

    :param lhs: lhs of the subtraction, ``[- 2k * math.pi, 2k * math.pi]``
    :param rhs: rhs of the subtraction, ``[- 2k * math.pi, 2k * math.pi]``
    :return: angle difference within range of ``[-math.pi, math.pi)``.
    """
    diff = math.fmod(lhs - rhs, TWO_PI)
    angle_diff = np.arctan2(np.sin(diff), np.cos(diff))
    return angle_diff


class Interval:
    def __init__(self, start: Union[int, float], end: Union[int, float]):
        self._start = None
        self._end = None
        self.start = start
        self.end = end

    def __eq__(self, other):
        if not isinstance(other, Interval):
            warnings.warn(f"Inequality between Interval {repr(self)} and different type {type(other)}")
            return False

        return self._start == other.start and self._end == other.end

    def __hash__(self):
        return hash((self._start, self._end))

    def __iter__(self):
        yield self._start
        yield self._end

    def __truediv__(self, other) -> 'Interval':
        if other > 0.0:
            return type(self)(self._start/other, self._end/other)
        else:
            return type(self)(self._end / other, self._start / other)

    def __mul__(self, other: Union[int, float]) -> 'Interval':
        if other > 0.0:
            return type(self)(self._start * other, self._end * other)
        else:
            return type(self)(self._end * other, self._start * other)

    def __round__(self, n=None) -> 'Interval':
        return type(self)(round(self._start, n), round(self._end, n))

    def __add__(self, other: Union[float, int]) -> 'Interval':
        return type(self)(self._start + other, self._end + other)

    def __sub__(self, other: Union[float, int]) -> 'Interval':
        return type(self)(self._start - other, self._end - other)

    @property
    def start(self) -> Union[int, float]:
        return self._start

    @start.setter
    def start(self, start: Union[int, float]):
        if self._end is not None:
            assert start <= self._end, '<common.util/Interval> start of interval must be <= end, ' \
                                       'but start {} > end {}'.format(start, self._end)
        self._start = start

    @property
    def end(self) -> Union[int, float]:
        return self._end

    @end.setter
    def end(self, end: Union[int, float]):
        if self._start is not None:
            assert end >= self._start, '<common.util/Interval> start of interval must be <= end, ' \
                                       'but start {} > end {}'.format(self._start, end)
        self._end = end

    def contains(self, other: Union[int, float, 'Interval']) -> bool:
        if type(other) is Interval:
            return self.start <= other.start and other.end <= self.end
        else:
            return self.start <= other <= self.end

    def __contains__(self, value: Union[int, float, "Interval"]):
        return self.contains(value)

    def overlaps(self, interval: 'Interval') -> bool:
        return self.end >= interval.start and interval.end >= self.start

    def intersection(self, other: 'Interval') -> Union[None, 'Interval']:
        """
        Returns the intersections with another interval.
        :param other: Interval
        :return: Interval or None if intersection is empty
        """
        if not self.overlaps(other):
            return None
        else:
            return Interval(max(self._start, other._start), min(self._end, other._end))

    @property
    def length(self):
        return self.end - self.start

    def __str__(self):
        info_str = "Interval\n"
        info_str += "start: {}\n".format(self._start)
        info_str += "end: {}\n".format(self._end)
        return info_str

    def __gt__(self, other):
        if isinstance(other, validity.ValidTypes.NUMBERS):
            return True if self._start > other else False
        else:
            return True if self._start > other.end else False

    def __lt__(self, other):
        if isinstance(other, validity.ValidTypes.NUMBERS):
            return True if self._end < other else False
        else:
            return True if self._end < other.start else False


class AngleInterval(Interval):
    """ Allows only angles from interval [-2pi,2pi]"""
    def __init__(self, start: Union[int, float], end: Union[int, float]):
        start, end = make_valid_orientation_interval(start, end)
        assert end - start < TWO_PI, '<common.util/AngleInterval> Interval must not be |start-end| > 2pi'
        Interval.__init__(self, start, end)

    @property
    def start(self) -> Union[int, float]:
        return self._start

    @start.setter
    def start(self, start: Union[int, float]):
        assert is_valid_orientation(start), '<common.util/AngleInterval> start angle needs to be in interval [-2pi,2pi]'
        if self._end is not None:
            assert start <= self._end, '<common.util/Interval> start of interval must be <= end, ' \
                                       'but start {} > end {}'.format(start, self._end)
        self._start = start

    @property
    def end(self) -> Union[int, float]:
        return self._end

    @end.setter
    def end(self, end: Union[int, float]):
        assert is_valid_orientation(end), '<common.util/AngleInterval> end angle needs to be in interval [-2pi,2pi]'
        if self._start is not None:
            assert end >= self._start, '<common.util/Interval> start of interval must be <= end, ' \
                                       'but start {} > end {}'.format(self._start, end)
        self._end = end

    def intersect(self, other: 'AngleInterval'):
        raise NotImplementedError()

    def contains(self, other: Union[float, 'AngleInterval']) -> bool:
        if isinstance(other, float):
            return self.__contains__(other)
        interval_diff = vectorized_angle_difference(self.end, self.start)
        assert interval_diff >= 0
        start_diff = vectorized_angle_difference(other.start, self.start)
        end_diff = vectorized_angle_difference(other.end, self.start)
        return 0 <= start_diff <= interval_diff and 0 <= end_diff <= interval_diff

    def __contains__(self, value: Union[int, float]):
        interval_diff = vectorized_angle_difference(self.end, self.start)
        assert interval_diff >= 0
        diff = vectorized_angle_difference(value, self.start)
        return 0 <= diff <= interval_diff


class FileFormat(enum.Enum):
    """
    Specifies the format of file.
    """
    XML = ".xml"
    PROTOBUF = ".pb"


class Time:
    """
    Class which describes the fictive time when a scenario starts.
    """

    def __init__(self, hours: int, minutes: int, day: int = None, month: int = None, year: int = None):
        """
        Constructor of a time object

        :param hours: hours at start of scenario (0-24)
        :param minutes: minutes at start of scenario (0-60)
        :param day: day at start of scenario (1-31)
        :param month: month at start of scenario (1-12)
        :param year: year at start of scenario
        """
        self._hours = hours
        self._minutes = minutes
        self._day = day
        self._month = month
        self._year = year

    def __eq__(self, other):
        if not isinstance(other, Time):
            return False

        return self._hours == other.hours and self._minutes == other.minutes and self._day == other.day and \
            self._month == other.month and self._year == other.year

    def __str__(self):
        return f"Year {self._year}, month {self._month}, day {self._day}, hour {self._hours}, minute {self.minutes}"

    def __hash__(self):
        return hash((self._hours, self._minutes, self._day, self._month, self._year))

    @property
    def hours(self) -> int:
        """ Hours at start of scenario (0-24)"""
        return self._hours

    @hours.setter
    def hours(self, hours: int):
        self._hours = hours

    @property
    def minutes(self) -> int:
        """ Minutes at start of scenario (0-60)"""
        return self._minutes

    @minutes.setter
    def minutes(self, minutes: int):
        self._minutes = minutes

    @property
    def day(self) -> Union[None, int]:
        """ Day at start of scenario (1-31)"""
        return self._day

    @day.setter
    def day(self, day: Union[None, int]):
        self._day = day

    @property
    def month(self) -> Union[None, int]:
        """ Month at start of scenario (1-12)"""
        return self._month

    @month.setter
    def month(self, month: Union[None, int]):
        self._month = month

    @property
    def year(self) -> Union[None, int]:
        """ Year at start of scenario"""
        return self._year

    @year.setter
    def year(self, year: Union[None, int]):
        self._year = year




import os
import shutil
from os.path import isfile, join, isdir

from commonroad.common.file_reader import CommonRoadFileReader
from commonroad.common.file_writer import CommonRoadFileWriter
from commonroad.common.util import FileFormat
from commonroad.common.writer.file_writer_interface import OverwriteExistingFile


def convert_xml_to_pb(src_file_path: str, dest_file_path: str, invalid_conversion_path: str = None):
    """
    Converts CommonRoad file from XML to protobuf format. Invalid converted files can be stored in a desired directory.

    :param src_file_path: Source path
    :param dest_file_path: Destination path
    :param invalid_conversion_path: Invalid conversion path
    :return:
    """
    assert os.path.exists(src_file_path), 'The path to source directory is not existent!'
    assert src_file_path.endswith('.xml'), 'No XML file!'
    assert dest_file_path.endswith('.pb'), 'No protobuf file!'
    assert invalid_conversion_path is None or os.path.exists(invalid_conversion_path), \
        'The path to invalid conversion directory is not existent!'

    try:
        scenario_xml, planning_problem_set_xml = CommonRoadFileReader(src_file_path, FileFormat.XML) \
            .open(lanelet_assignment=True)
    except Exception:
        return

    CommonRoadFileWriter(scenario_xml, planning_problem_set_xml, file_format=FileFormat.PROTOBUF) \
        .write_to_file(dest_file_path, OverwriteExistingFile.ALWAYS)

    if invalid_conversion_path is not None:
        scenario_pb, planning_problem_set_pb = CommonRoadFileReader(dest_file_path, FileFormat.PROTOBUF) \
            .open(lanelet_assignment=True)

        if not (scenario_xml == scenario_pb and planning_problem_set_xml == planning_problem_set_pb):
            path_splits = src_file_path.split('/')
            if path_splits:
                name = path_splits[-1]
            else:
                return

            invalid_conversion_file_path = invalid_conversion_path + "/" + name

            CommonRoadFileWriter(scenario_xml, planning_problem_set_xml, file_format=FileFormat.XML) \
                .write_to_file(invalid_conversion_file_path, OverwriteExistingFile.ALWAYS)

    print("Conversion of " + str(scenario_xml.scenario_id))


def convert_xml_to_pb_in_dirs(src_dir_path: str, dest_dir_path: str, invalid_conversion_path: str = None):
    """
    Converts all CommonRoad files in directory and its subdirectories from XML to protobuf format.
    The hierarchical file structure is copied to the destination directory. Non CommonRoad file are only copied.

    :param src_dir_path: Source directory
    :param dest_dir_path: Destination directory
    :param invalid_conversion_path: Invalid conversion path
    :return:
    """
    assert os.path.exists(src_dir_path), 'The path to source directory is not existent!'
    assert os.path.exists(dest_dir_path), 'The path to destination directory is not existent!'
    assert invalid_conversion_path is None or os.path.exists(invalid_conversion_path), \
        'The path to invalid conversion directory is not existent!'

    valid_xml_file_names = list()
    invalid_xml_file_names = list()
    dir_names = list()
    for name in os.listdir(src_dir_path):

        if isfile(join(src_dir_path, name)):
            is_invalid_xml_file = False
            for ending in ['.add.xml', '.net.xml', '.rou.xml']:
                if name.endswith(ending) or not name.endswith('.xml'):
                    is_invalid_xml_file = True

            # if not is_invalid_xml_file:
            #     content = open(join(src_dir_path, name), 'rb').read()
            #     if not CommonRoadFileWriter.check_validity_of_commonroad_file(content, FileFormat.XML):
            #         is_invalid_xml_file = True

            if is_invalid_xml_file:
                invalid_xml_file_names.append(name)
            else:
                valid_xml_file_names.append(name)
        elif isdir(join(src_dir_path, name)):
            dir_names.append(name)

    for name in valid_xml_file_names:
        src_file_path = join(src_dir_path, name)
        dest_file_path = join(dest_dir_path, name.replace('.xml', '.pb'))

        convert_xml_to_pb(src_file_path, dest_file_path, invalid_conversion_path)

    for name in invalid_xml_file_names:
        src_file_path = join(src_dir_path, name)
        dest_file_path = join(dest_dir_path, name)

        shutil.copy(src_file_path, dest_file_path)

    for name in dir_names:
        src_sub_dir_path = join(src_dir_path, name)
        dest_sub_dir_path = join(dest_dir_path, name)

        if not os.path.exists(dest_sub_dir_path):
            os.mkdir(dest_sub_dir_path)

        convert_xml_to_pb_in_dirs(src_sub_dir_path, dest_sub_dir_path, invalid_conversion_path)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: obstacle.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='obstacle.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0eobstacle.proto\x12\ncommonroad\x1a\nutil.proto\"\x8f\x02\n\x10ObstacleTypeEnum\"\xfa\x01\n\x0cObstacleType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03\x43\x41R\x10\x01\x12\t\n\x05TRUCK\x10\x02\x12\x07\n\x03\x42US\x10\x03\x12\x0b\n\x07\x42ICYCLE\x10\x04\x12\x0e\n\nPEDESTRIAN\x10\x05\x12\x14\n\x10PRIORITY_VEHICLE\x10\x06\x12\x12\n\x0ePARKED_VEHICLE\x10\x07\x12\x15\n\x11\x43ONSTRUCTION_ZONE\x10\x08\x12\t\n\x05TRAIN\x10\t\x12\x11\n\rROAD_BOUNDARY\x10\n\x12\x0e\n\nMOTORCYCLE\x10\x0b\x12\x08\n\x04TAXI\x10\x0c\x12\x0c\n\x08\x42UILDING\x10\r\x12\n\n\x06PILLAR\x10\x0e\x12\x10\n\x0cMEDIAN_STRIP\x10\x0f\"\xd8\x11\n\x05State\x12\"\n\x05point\x18\x01 \x01(\x0b\x32\x11.commonroad.PointH\x00\x12\"\n\x05shape\x18\x02 \x01(\x0b\x32\x11.commonroad.ShapeH\x00\x12\x35\n\x0borientation\x18\x03 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x32\n\x08velocity\x18\x04 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0esteering_angle\x18\x05 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12>\n\x14steering_angle_speed\x18\x06 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x32\n\x08yaw_rate\x18\x07 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nslip_angle\x18\x08 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nroll_angle\x18\t \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\troll_rate\x18\n \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x35\n\x0bpitch_angle\x18\x0b \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\npitch_rate\x18\x0c \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nvelocity_y\x18\r \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nposition_z\x18\x0e \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nvelocity_z\x18\x0f \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10roll_angle_front\x18\x10 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0froll_rate_front\x18\x11 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10velocity_y_front\x18\x12 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10position_z_front\x18\x13 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10velocity_z_front\x18\x14 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0froll_angle_rear\x18\x15 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0eroll_rate_rear\x18\x16 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fvelocity_y_rear\x18\x17 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fposition_z_rear\x18\x18 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fvelocity_z_rear\x18\x19 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x43\n\x19\x66ront_wheel_angular_speed\x18\x1a \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x42\n\x18rear_wheel_angular_speed\x18\x1b \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12H\n\x1eleft_front_wheel_angular_speed\x18\x1c \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12I\n\x1fright_front_wheel_angular_speed\x18\x1d \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12G\n\x1dleft_rear_wheel_angular_speed\x18\x1e \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12H\n\x1eright_rear_wheel_angular_speed\x18\x1f \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tdelta_y_f\x18  \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tdelta_y_r\x18! \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x36\n\x0c\x61\x63\x63\x65leration\x18\" \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0e\x61\x63\x63\x65leration_y\x18# \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12.\n\x04jerk\x18$ \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tcurvature\x18% \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0e\x63urvature_rate\x18& \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x35\n\ttime_step\x18\' \x02(\x0b\x32\".commonroad.IntegerExactOrIntervalB\n\n\x08position\"\xd8\x01\n\x0bSignalState\x12\x35\n\ttime_step\x18\x01 \x01(\x0b\x32\".commonroad.IntegerExactOrInterval\x12\x0c\n\x04horn\x18\x02 \x01(\x08\x12\x16\n\x0eindicator_left\x18\x03 \x01(\x08\x12\x17\n\x0findicator_right\x18\x04 \x01(\x08\x12\x16\n\x0e\x62raking_lights\x18\x05 \x01(\x08\x12\x1d\n\x15hazard_warning_lights\x18\x06 \x01(\x08\x12\x1c\n\x14\x66lashing_blue_lights\x18\x07 \x01(\x08\"d\n\tOccupancy\x12\x35\n\ttime_step\x18\x01 \x02(\x0b\x32\".commonroad.IntegerExactOrInterval\x12 \n\x05shape\x18\x02 \x02(\x0b\x32\x11.commonroad.Shape\":\n\x0cOccupancySet\x12*\n\x0boccupancies\x18\x01 \x03(\x0b\x32\x15.commonroad.Occupancy\"J\n\nTrajectory\x12\x19\n\x11initial_time_step\x18\x01 \x02(\r\x12!\n\x06states\x18\x02 \x03(\x0b\x32\x11.commonroad.State\"d\n\x14TrajectoryPrediction\x12*\n\ntrajectory\x18\x01 \x02(\x0b\x32\x16.commonroad.Trajectory\x12 \n\x05shape\x18\x02 \x02(\x0b\x32\x11.commonroad.Shape\"`\n\x12SetBasedPrediction\x12\x19\n\x11initial_time_step\x18\x01 \x02(\r\x12/\n\roccupancy_set\x18\x02 \x02(\x0b\x32\x18.commonroad.OccupancySet'
  ,
  dependencies=[util__pb2.DESCRIPTOR,])



_OBSTACLETYPEENUM_OBSTACLETYPE = _descriptor.EnumDescriptor(
  name='ObstacleType',
  full_name='commonroad.ObstacleTypeEnum.ObstacleType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRUCK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PEDESTRIAN', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRIORITY_VEHICLE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARKED_VEHICLE', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONSTRUCTION_ZONE', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAIN', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROAD_BOUNDARY', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MOTORCYCLE', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TAXI', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUILDING', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PILLAR', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIAN_STRIP', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=64,
  serialized_end=314,
)
_sym_db.RegisterEnumDescriptor(_OBSTACLETYPEENUM_OBSTACLETYPE)


_OBSTACLETYPEENUM = _descriptor.Descriptor(
  name='ObstacleTypeEnum',
  full_name='commonroad.ObstacleTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OBSTACLETYPEENUM_OBSTACLETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=314,
)


_STATE = _descriptor.Descriptor(
  name='State',
  full_name='commonroad.State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='point', full_name='commonroad.State.point', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.State.shape', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orientation', full_name='commonroad.State.orientation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='commonroad.State.velocity', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='steering_angle', full_name='commonroad.State.steering_angle', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='steering_angle_speed', full_name='commonroad.State.steering_angle_speed', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='yaw_rate', full_name='commonroad.State.yaw_rate', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slip_angle', full_name='commonroad.State.slip_angle', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle', full_name='commonroad.State.roll_angle', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate', full_name='commonroad.State.roll_rate', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pitch_angle', full_name='commonroad.State.pitch_angle', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pitch_rate', full_name='commonroad.State.pitch_rate', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y', full_name='commonroad.State.velocity_y', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z', full_name='commonroad.State.position_z', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z', full_name='commonroad.State.velocity_z', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle_front', full_name='commonroad.State.roll_angle_front', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate_front', full_name='commonroad.State.roll_rate_front', index=16,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y_front', full_name='commonroad.State.velocity_y_front', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z_front', full_name='commonroad.State.position_z_front', index=18,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z_front', full_name='commonroad.State.velocity_z_front', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle_rear', full_name='commonroad.State.roll_angle_rear', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate_rear', full_name='commonroad.State.roll_rate_rear', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y_rear', full_name='commonroad.State.velocity_y_rear', index=22,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z_rear', full_name='commonroad.State.position_z_rear', index=23,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z_rear', full_name='commonroad.State.velocity_z_rear', index=24,
      number=25, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='front_wheel_angular_speed', full_name='commonroad.State.front_wheel_angular_speed', index=25,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rear_wheel_angular_speed', full_name='commonroad.State.rear_wheel_angular_speed', index=26,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_front_wheel_angular_speed', full_name='commonroad.State.left_front_wheel_angular_speed', index=27,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_front_wheel_angular_speed', full_name='commonroad.State.right_front_wheel_angular_speed', index=28,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_rear_wheel_angular_speed', full_name='commonroad.State.left_rear_wheel_angular_speed', index=29,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_rear_wheel_angular_speed', full_name='commonroad.State.right_rear_wheel_angular_speed', index=30,
      number=31, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta_y_f', full_name='commonroad.State.delta_y_f', index=31,
      number=32, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta_y_r', full_name='commonroad.State.delta_y_r', index=32,
      number=33, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acceleration', full_name='commonroad.State.acceleration', index=33,
      number=34, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acceleration_y', full_name='commonroad.State.acceleration_y', index=34,
      number=35, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jerk', full_name='commonroad.State.jerk', index=35,
      number=36, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='curvature', full_name='commonroad.State.curvature', index=36,
      number=37, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='curvature_rate', full_name='commonroad.State.curvature_rate', index=37,
      number=38, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.State.time_step', index=38,
      number=39, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='position', full_name='commonroad.State.position',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=317,
  serialized_end=2581,
)


_SIGNALSTATE = _descriptor.Descriptor(
  name='SignalState',
  full_name='commonroad.SignalState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.SignalState.time_step', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='horn', full_name='commonroad.SignalState.horn', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indicator_left', full_name='commonroad.SignalState.indicator_left', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indicator_right', full_name='commonroad.SignalState.indicator_right', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='braking_lights', full_name='commonroad.SignalState.braking_lights', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hazard_warning_lights', full_name='commonroad.SignalState.hazard_warning_lights', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flashing_blue_lights', full_name='commonroad.SignalState.flashing_blue_lights', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2584,
  serialized_end=2800,
)


_OCCUPANCY = _descriptor.Descriptor(
  name='Occupancy',
  full_name='commonroad.Occupancy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.Occupancy.time_step', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.Occupancy.shape', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2802,
  serialized_end=2902,
)


_OCCUPANCYSET = _descriptor.Descriptor(
  name='OccupancySet',
  full_name='commonroad.OccupancySet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='occupancies', full_name='commonroad.OccupancySet.occupancies', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2904,
  serialized_end=2962,
)


_TRAJECTORY = _descriptor.Descriptor(
  name='Trajectory',
  full_name='commonroad.Trajectory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_time_step', full_name='commonroad.Trajectory.initial_time_step', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='states', full_name='commonroad.Trajectory.states', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2964,
  serialized_end=3038,
)


_TRAJECTORYPREDICTION = _descriptor.Descriptor(
  name='TrajectoryPrediction',
  full_name='commonroad.TrajectoryPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trajectory', full_name='commonroad.TrajectoryPrediction.trajectory', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.TrajectoryPrediction.shape', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3040,
  serialized_end=3140,
)


_SETBASEDPREDICTION = _descriptor.Descriptor(
  name='SetBasedPrediction',
  full_name='commonroad.SetBasedPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_time_step', full_name='commonroad.SetBasedPrediction.initial_time_step', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='occupancy_set', full_name='commonroad.SetBasedPrediction.occupancy_set', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3142,
  serialized_end=3238,
)

_OBSTACLETYPEENUM_OBSTACLETYPE.containing_type = _OBSTACLETYPEENUM
_STATE.fields_by_name['point'].message_type = util__pb2._POINT
_STATE.fields_by_name['shape'].message_type = util__pb2._SHAPE
_STATE.fields_by_name['orientation'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['steering_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['steering_angle_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['yaw_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['slip_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['pitch_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['pitch_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['left_front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['right_front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['left_rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['right_rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['delta_y_f'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['delta_y_r'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['acceleration'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['acceleration_y'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['jerk'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['curvature'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['curvature_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_STATE.oneofs_by_name['position'].fields.append(
  _STATE.fields_by_name['point'])
_STATE.fields_by_name['point'].containing_oneof = _STATE.oneofs_by_name['position']
_STATE.oneofs_by_name['position'].fields.append(
  _STATE.fields_by_name['shape'])
_STATE.fields_by_name['shape'].containing_oneof = _STATE.oneofs_by_name['position']
_SIGNALSTATE.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_OCCUPANCY.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_OCCUPANCY.fields_by_name['shape'].message_type = util__pb2._SHAPE
_OCCUPANCYSET.fields_by_name['occupancies'].message_type = _OCCUPANCY
_TRAJECTORY.fields_by_name['states'].message_type = _STATE
_TRAJECTORYPREDICTION.fields_by_name['trajectory'].message_type = _TRAJECTORY
_TRAJECTORYPREDICTION.fields_by_name['shape'].message_type = util__pb2._SHAPE
_SETBASEDPREDICTION.fields_by_name['occupancy_set'].message_type = _OCCUPANCYSET
DESCRIPTOR.message_types_by_name['ObstacleTypeEnum'] = _OBSTACLETYPEENUM
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['SignalState'] = _SIGNALSTATE
DESCRIPTOR.message_types_by_name['Occupancy'] = _OCCUPANCY
DESCRIPTOR.message_types_by_name['OccupancySet'] = _OCCUPANCYSET
DESCRIPTOR.message_types_by_name['Trajectory'] = _TRAJECTORY
DESCRIPTOR.message_types_by_name['TrajectoryPrediction'] = _TRAJECTORYPREDICTION
DESCRIPTOR.message_types_by_name['SetBasedPrediction'] = _SETBASEDPREDICTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObstacleTypeEnum = _reflection.GeneratedProtocolMessageType('ObstacleTypeEnum', (_message.Message,), {
  'DESCRIPTOR' : _OBSTACLETYPEENUM,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ObstacleTypeEnum)
  })
_sym_db.RegisterMessage(ObstacleTypeEnum)

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), {
  'DESCRIPTOR' : _STATE,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.State)
  })
_sym_db.RegisterMessage(State)

SignalState = _reflection.GeneratedProtocolMessageType('SignalState', (_message.Message,), {
  'DESCRIPTOR' : _SIGNALSTATE,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.SignalState)
  })
_sym_db.RegisterMessage(SignalState)

Occupancy = _reflection.GeneratedProtocolMessageType('Occupancy', (_message.Message,), {
  'DESCRIPTOR' : _OCCUPANCY,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Occupancy)
  })
_sym_db.RegisterMessage(Occupancy)

OccupancySet = _reflection.GeneratedProtocolMessageType('OccupancySet', (_message.Message,), {
  'DESCRIPTOR' : _OCCUPANCYSET,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.OccupancySet)
  })
_sym_db.RegisterMessage(OccupancySet)

Trajectory = _reflection.GeneratedProtocolMessageType('Trajectory', (_message.Message,), {
  'DESCRIPTOR' : _TRAJECTORY,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Trajectory)
  })
_sym_db.RegisterMessage(Trajectory)

TrajectoryPrediction = _reflection.GeneratedProtocolMessageType('TrajectoryPrediction', (_message.Message,), {
  'DESCRIPTOR' : _TRAJECTORYPREDICTION,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TrajectoryPrediction)
  })
_sym_db.RegisterMessage(TrajectoryPrediction)

SetBasedPrediction = _reflection.GeneratedProtocolMessageType('SetBasedPrediction', (_message.Message,), {
  'DESCRIPTOR' : _SETBASEDPREDICTION,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.SetBasedPrediction)
  })
_sym_db.RegisterMessage(SetBasedPrediction)


# @@protoc_insertion_point(module_scope)
